/****************************************************************/
/*  uart.c 														*/
/*																*/
/*	handelt die beiden Schnittstellen							*/
/*																*/
/*	written by Wolfgang Weirczeyko	Mrz 2008					*/
/*																*/
/*	uart0 == Daten von der FC									*/			
/*	uart1 == Daten zum PC										*/
/****************************************************************/

#include "main.h"

unsigned char s_temp;
unsigned char umode0 = 0;
char C_cr = 0x0d;
char C_lf = 0x0a;
volatile unsigned char ds_komplett_0 = 0;

void init_uarts(void)
{

	
	UBRR0L = (uint8_t)34;		/* Baudrate auf 56,7		*/
	UBRR0H = (uint8_t)0;		/* Bei 16MHz				*/
	UBRR1L = (uint8_t)34;		/*    """"""				*/
	UBRR1H = (uint8_t)0;		/*    """"""				*/


	 						
	PORTD |= _BV(PD0); //Pullup aktivieren					
	PORTB |= _BV(PD2); //Pullup aktivieren	


	UCSR0B  = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0); 	/* enable RX, TX, interrupt RX */
	UCSR0C  = (1<<URSEL0)|(1<<UCSZ01) | (1<<UCSZ00);/* 8 Data, No parity, 1 Stop  	*/
	UCSR0A  = (1 << RXC0) | (1 << TXC0) | (1 << U2X0);

	UCSR1B  = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0); 	/* enable RX, TX, interrupt RX */
	UCSR1C  = (1<<URSEL0)|(1<<UCSZ01) | (1<<UCSZ00);/* 8 Data, No parity, 1 Stop  	*/
	UCSR1A  = (1 << RXC0) | (1 << TXC0) | (1 << U2X0);
}

void send_FC_C(void)
{

 unsigned char p[10];
 int len, crc,i;
  
  p[0]='#';
  p[1]=0;
  p[2]='c';
  p[3]=0;
  p[4]=0;
  len=5;
  crc=0;
  for (i=0;i<len;i++)
    crc+=p[i];
  
  crc%=4096;
  p[len++]= '=' + crc/64;
  p[len++]= '=' + crc%64;
  p[len++]= '\r';
  
  for(i=0;i< 8; i++)

  	uart_send_1( p[i]);
  
}



void uart_send_1(unsigned char z)					/* Noch nicht so schoen. Interupt wre besser */
{
	while ( !( UCSR1A & (1<<UDRE1)) )	;			/* Warten bis Sendebuffer leer */
	UDR1 = z;									/* Zeichen ausgeben */
}



void uart_send_0(unsigned char z)					/* Noch nicht so schoen. Interupt wre besser */
{
	while ( !( UCSR0A & (1<<UDRE0)) )	;			/* Warten bis Sendebuffer leer */
	UDR0 = z;									/* Zeichen ausgeben */
}




void uart_send_string_0(char *a)					/* Zum ausgeben von 0terminierten Strings */					
{
	unsigned int len, i;

	len = strlen(a);							/* Die Stringlnge rauskriegen */
	for(i=0;i<=len;i++)
		uart_send_0(a[i]);						/* und senden */

	uart_send_0(C_cr);
	uart_send_0(C_lf);							/* cr lf nur fr Terminal */

}

void uart_send_int_0( int k )
{
	char str[10];

	itoa( k, str, 10 ); 
	uart_send_string_0(str);

}

void uart_send_binary(unsigned char k)		/* Sende an PC den Byte im Binrformat  */
{
	 char i;
	for( i=7;i>=0;i--)						/* Bit 7 zuerst							*/
	{
		if ( (k & (1<<i)) > 0)				/* Wenn Bit == 1 dann eine "1" senden	*/	
			uart_send_0(0x31);				
		else	
			uart_send_0(0x30);
	}
	uart_send_0(C_cr);
	uart_send_0(C_lf);

}

SIGNAL(SIG_USART1_RECV)  				/* Byte auf USART1 ist da */
{

	s_temp = UDR1;  					/* Byte holen.	*/

	if ( (s_temp == '\r') && ( umode0 == 1 )) /* Endezeichen und Mode == 1 */
	{	
		ROT_OFF1;
		umode0 = 0; 					/* Neuer datensatz kommt gleich */
		ds_komplett_0 = 1 ;				/* Datensatz 0 ist da und kann verarbeitet werden.*/
		RX_BUFF_0[rx_poi_0] = s_temp;	/* Speichern im Buffer */ 
										/* Neuer Datensatz da kann verarbeitet werden */
	}
	else
	switch(umode0)
	{

		case 0:
			if ( (s_temp == '#') && ( !ds_komplett_0)) 	/* Startzeichen und Dazten schon verarbeitet */
			{
				umode0 = 1;								/* Modus Datensammeln */
				ROT_ON1;
			}
			rx_poi_0 = 0;					/* Pointer auf Null setzten. Neuer Datensatz */
										
			RX_BUFF_0[rx_poi_0++] = s_temp;	/* Byte in den Buffer */
			break;
		case 1: 		/* Daten sammeln */
			RX_BUFF_0[rx_poi_0] = s_temp;	/* Byte in den Buffer */

			if ( rx_poi_0 < RXB_LEN_0)		/* Buffer voll dann abbruch */
				rx_poi_0++;
			else
				umode0 = 0;					/* Fange von vorne an */
			break;

	}	

}

void send_debug_data(void)
{
	char sw[10];
	char str[20];

	itoa (Debug_Data.Analog[0],str,10);
	sw[0] = '@';
	sw[1] = 'D';
	sw[2] = '0';

	sw[3] = 0;
	strncat(sw,str,strlen(str));
	strncat(sw,"#",1);
	uart_send_string_0(sw);


	itoa (Debug_Data.Analog[1],str,10);
	sw[0] = '@';
	sw[1] = 'D';
	sw[2] = '1';

	sw[3] = 0;
	strncat(sw,str,strlen(str));
	strncat(sw,"#",1);
	uart_send_string_0(sw);

	itoa (Debug_Data.Analog[5],str,10);
	sw[0] = '@';
	sw[1] = 'h';
	sw[2] = '=';

	sw[3] = 0;
	strncat(sw,str,strlen(str));
	strncat(sw,"#",1);
	uart_send_string_0(sw);

	itoa (Debug_Data.Analog[9],str,10);
	sw[0] = '@';
	sw[1] = 'V';
	sw[2] = 'l';

	sw[3] = 0;
	strncat(sw,str,strlen(str));
	strncat(sw,"#",1);
	uart_send_string_0(sw);
//	uart_send_0(C_cr);
//	uart_send_0(C_lf);

	itoa (K_GRAD,str,10);
	sw[0] = '@';
	sw[1] = 'K';
	sw[2] = 'g';

	sw[3] = 0;
	strncat(sw,str,strlen(str));
	strncat(sw,"#",1);
	uart_send_string_0(sw);
//	uart_send_0(C_cr);
//	uart_send_0(C_lf);


}



